#include "stdafx.h"
#include "InstallUtils.h"
#include "KernelCall.h"

namespace
{
    const LPCWSTR DriverName = L"Sample";
    void Install( TCHAR* arg ) 
    {
        if (_tcscmp(arg, TEXT("--install")) == 0)
        {
            wchar_t buffer[MAX_PATH];
            if (0 == ::GetModuleFileNameW(NULL, buffer, MAX_PATH))
            {
                throw std::runtime_error("Unable to get module file name");
            }
            std::wstring curDir(buffer);
            curDir = curDir.substr(0, curDir.find_last_of(L"\\") + 1);
            InstallDriver(curDir + DriverName + L".sys", DriverName);
        }
        else if (_tcscmp(arg, TEXT("--uninstall")) == 0)
        {
            UninstallDriver(DriverName);
        }
        else
        {
            throw std::runtime_error("Invalid argument");
        }
    }
}

int g_tempForAddress;

void PrintPhysicalAddresses() 
{
    std::cout << std::hex << "source: " << "\t\t" << "virtual  " << "\t" << "physical  " << std::endl << std::endl;
    SampleCall sample;
    
    LONGLONG physicalAddress = sample.VirtualToPhysicalAddr(&g_tempForAddress);
    std::cout << std::hex << "global variable:" << "\t" << &g_tempForAddress << "\t" << std::setfill ('0') << std::setw (16) << std::uppercase << physicalAddress << std::endl;

    int tempForAddress;
    physicalAddress = sample.VirtualToPhysicalAddr(&tempForAddress);
    std::cout << std::hex << "stack variable: " << "\t" << &tempForAddress << "\t" << std::setfill ('0') << std::setw (16) << std::uppercase << physicalAddress << std::endl;

    physicalAddress = sample.VirtualToPhysicalAddr(GetModuleHandleA("ntdll.dll"));
    std::cout << std::hex << "ntdll.dll:" << "\t\t" << GetModuleHandleA("ntdll.dll") << "\t" << std::setfill ('0') << std::setw (16) << std::uppercase << physicalAddress << std::endl;

    physicalAddress = sample.VirtualToPhysicalAddr(GetModuleHandleA("kernel32.dll"));
    std::cout << std::hex << "kernel32.dll:" << "\t\t" << GetModuleHandleA("kernel32.dll") << "\t" << std::setfill ('0') << std::setw (16) << std::uppercase << physicalAddress << std::endl;
}

int _tmain(int /*argc*/, _TCHAR* argv[])
{
    try
    {
        if (NULL == argv[1])
        {
            PrintPhysicalAddresses();
        }
        else
        {
            Install(argv[1]);
        }
    }
    catch(const std::exception& ex)
    {
        std::cout << ex.what() << std::endl;
        return -1;
    }
    catch (...)
    {
        std::cout << "UNHANDLED EXCEPTION!!!" << std::endl;
        return -1;
    }

	return 0;
}

