#include "stdafx.h"
#include "ScUtils.h"

SC_HANDLE sc_utils::OpenManager()
{
    SC_HANDLE manager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
    if( NULL == manager )
    {
        throw std::runtime_error("Unable to open service control manager");
    }
    return manager;
}

SC_HANDLE sc_utils::OpenDriverAsService( const std::wstring& driverName, DWORD desiredAcess )
{
    ScHandleGuard manager(OpenManager());
    SC_HANDLE service = OpenServiceW(manager.get(), driverName.c_str(), desiredAcess);
    if (NULL == service)
    {
        throw std::runtime_error("Unable to open service");
    }
    return service;
}

HANDLE sc_utils::OpenDriverAsFile( const std::wstring& driverName )
{
    HANDLE driver = CreateFileW(
        driverName.c_str(), 
        GENERIC_WRITE | GENERIC_READ,
        FILE_SHARE_READ | FILE_SHARE_WRITE, 
        NULL,
        OPEN_EXISTING, 
        0, 
        NULL);
    if (INVALID_HANDLE_VALUE == driver)
    {
        throw std::runtime_error("Unable to open driver");
    }
    return driver;
}

sc_utils::DriverStarter::DriverStarter( const std::wstring& driverName )
    : m_service(OpenDriverAsService(driverName, SERVICE_START | SERVICE_STOP))
{
    if ((!::StartService(m_service.get(), 0, NULL)) && (ERROR_SERVICE_ALREADY_RUNNING != ::GetLastError()))
    {
        throw std::runtime_error("Unable to start service");
    }
}

sc_utils::DriverStarter::~DriverStarter()
{
    SERVICE_STATUS status;
    if (!ControlService(m_service.get(), SERVICE_CONTROL_STOP, &status))
    {
    }
}
