#pragma once

namespace sc_utils
{
    SC_HANDLE OpenManager();
    SC_HANDLE OpenDriverAsService(const std::wstring& driverName, DWORD desiredAcess);
    HANDLE OpenDriverAsFile(const std::wstring& driverName);

    class ScHandleGuard
    {
    public:
        ScHandleGuard(SC_HANDLE h)
            : m_handle(h)
        {
        }

        SC_HANDLE get()
        {
            return m_handle;
        }

        ~ScHandleGuard()
        {
            if (NULL != m_handle)
                CloseServiceHandle(m_handle);
        }

    private:
        ScHandleGuard(const ScHandleGuard&);
        ScHandleGuard& operator = (const ScHandleGuard&);

    private:
        SC_HANDLE m_handle;
    };

    class HandleGuard
    {
    public:
        HandleGuard(HANDLE h)
            : m_handle(h)
        {
        }

        HANDLE get()
        {
            return m_handle;
        }

        ~HandleGuard()
        {
            if (INVALID_HANDLE_VALUE != m_handle)
                CloseHandle(m_handle);
        }

    private:
        HandleGuard(const HandleGuard&);
        HandleGuard& operator = (const HandleGuard&);

    private:
        HANDLE m_handle;
    };

    class DriverStarter
    {
    public:
        DriverStarter(const std::wstring& driverName);
        ~DriverStarter();

    private:
        ScHandleGuard m_service;
    };
}