extern "C"
{
#include "ntifs.h"
}

#include "libcpp.h"
#include "new"
#include "stdexcept"

#include "Sample.h"
#include "IoControl.h"

//////////////////////////////////////////////////////////////////////////

typedef rpc::DefaultExceptions Exceptions;

rpc::RpcServer<Exceptions>* g_RpcServer = NULL;
Sample* g_SampleObject = NULL;

typedef struct SAMPLE_DEVICE_EXTENSION
{
    PDEVICE_OBJECT	fdo;
    UNICODE_STRING	SymLinkName;
} *PSAMPLE_DEVICE_EXTENSION;


//////////////////////////////////////////////////////////////////////////

NTSTATUS RpcProcess(char* buf, size_t inSize, size_t outSize, size_t* written)
{
    try
    {
        g_RpcServer->Process(buf, inSize, outSize, &outSize);
        *written = outSize;
        return STATUS_SUCCESS;
    }
    catch(const std::exception& ex)
    {
        KdPrint((__FUNCTION__" [RPC CALL] %s\n", ex.what()));
        return STATUS_UNSUCCESSFUL;
    }
}

//////////////////////////////////////////////////////////////////////////

NTSTATUS DrvCreate(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
    NTSTATUS status = STATUS_SUCCESS;

    Irp->IoStatus.Information = 0;
    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);

    return status;
}
NTSTATUS DrvClose(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
    NTSTATUS status = STATUS_SUCCESS;

    Irp->IoStatus.Information = 0;
    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);

    return status;
}

NTSTATUS DrvDeviceIoControl(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
    PIO_STACK_LOCATION irpStack = IoGetCurrentIrpStackLocation(Irp);
    ULONG code = irpStack->Parameters.DeviceIoControl.IoControlCode;
    
    if (code == RPC_CALL_IOCTL)
    {
        size_t inBufSize = irpStack->Parameters.DeviceIoControl.InputBufferLength;
        size_t outBufSize = irpStack->Parameters.DeviceIoControl.OutputBufferLength;
        size_t outSizeRes = 0;
        NTSTATUS status = RpcProcess((char*)Irp->AssociatedIrp.SystemBuffer,
            inBufSize, outBufSize, &outSizeRes);
        Irp->IoStatus.Information = outSizeRes;
        IoCompleteRequest(Irp, IO_NO_INCREMENT);

        return status;
    }
    else
    {
        NTSTATUS status = STATUS_INVALID_DEVICE_REQUEST;
        Irp->IoStatus.Status = status;
        IoCompleteRequest(Irp, IO_NO_INCREMENT);

        return status;
    }
}

//////////////////////////////////////////////////////////////////////////

static void DrvUnload(PDRIVER_OBJECT driverObject)
{
    PDEVICE_OBJECT deviceObject = driverObject->DeviceObject;

    while (NULL != deviceObject)
    {
        PSAMPLE_DEVICE_EXTENSION dx = (PSAMPLE_DEVICE_EXTENSION)deviceObject->DeviceExtension;

        deviceObject = deviceObject->NextDevice;

        IoDeleteSymbolicLink(&dx->SymLinkName);
        IoDeleteDevice(dx->fdo);
    }


    delete g_SampleObject;
    delete g_RpcServer;

    libcpp_exit();
}

extern "C" NTSTATUS DriverEntry(PDRIVER_OBJECT driverObject, PUNICODE_STRING registryPath)
{
    libcpp_init();

    driverObject->DriverUnload = DrvUnload;

    driverObject->MajorFunction[IRP_MJ_CREATE] = DrvCreate;
    driverObject->MajorFunction[IRP_MJ_CLOSE] = DrvClose;
    driverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = DrvDeviceIoControl;

    UNICODE_STRING devName;
    RtlInitUnicodeString( &devName, L"\\Device\\Sample" );

    PDEVICE_OBJECT  fdo;
    NTSTATUS status = IoCreateDevice(driverObject, sizeof(SAMPLE_DEVICE_EXTENSION), &devName, FILE_DEVICE_UNKNOWN, 0, FALSE, &fdo);
    if(!NT_SUCCESS(status)) 
        return status;

    PSAMPLE_DEVICE_EXTENSION dx = (PSAMPLE_DEVICE_EXTENSION)fdo->DeviceExtension;
    dx->fdo = fdo;

    UNICODE_STRING symLinkName;
    RtlInitUnicodeString(&symLinkName, L"\\DosDevices\\Sample");
    dx->SymLinkName = symLinkName;


    status = IoCreateSymbolicLink( &symLinkName, &devName );
    if (!NT_SUCCESS(status))
    {
        IoDeleteDevice( fdo );
        return status;
    }

    g_RpcServer = new rpc::RpcServer<Exceptions>;
    g_SampleObject = new Sample(g_RpcServer);

    return STATUS_SUCCESS;
}
