#include "RpcArchive.h"

//////////////////////////////////////////////////////////////////////////

rpc::Archive::Archive(char* buf, size_t bufSize, size_t used, size_t curRange)
: m_rangePtr(curRange)
, m_usedSize(used)
, m_buf(buf)
, m_bufSize(bufSize)
{
}

//////////////////////////////////////////////////////////////////////////

void rpc::Archive::Actualize()
{
    m_usedSize = GetSize();
}

size_t rpc::Archive::GetCurRange()
{
    return m_rangePtr;
}

//////////////////////////////////////////////////////////////////////////

char * rpc::Archive::TailAllocate(size_t size)
{
    size_t newSize =  m_usedSize + size;

    if (newSize > GetSize())
        throw std::runtime_error(std::string(__FUNCTION__) + " can't reallocate buffer.");

    char * lResult = GetPtr() + m_usedSize;
    m_usedSize = newSize;

    return lResult;
}

void rpc::Archive::UnTail(size_t size)
{
    if(m_usedSize < size)
        throw std::runtime_error(std::string(__FUNCTION__) + " can't untail - buffer empty.");
    m_usedSize -= size;
}

char * rpc::Archive::FetchRange(size_t size)
{
    if (m_rangePtr + size>m_usedSize)
        throw std::runtime_error("Buffer is out of range");

    size_t oldRangePtr = m_rangePtr;
    m_rangePtr += size;

    return GetPtr() +  oldRangePtr;
}

char * rpc::Archive::FetchRange(size_t length, size_t size)
{
    if (length + size > m_usedSize)
        throw std::runtime_error("Buffer is out of range");

    return GetPtr() + length;
}

size_t rpc::Archive::GetUsedSize() const
{
    return m_usedSize;
}

size_t rpc::Archive::GetSize() const
{
    return m_bufSize;
}

//////////////////////////////////////////////////////////////////////////

char * rpc::Archive::GetPtr()
{
    return m_buf;
}

const char * rpc::Archive::GetPtr() const
{
    return (const char * )m_buf;
}
