#ifndef RPC_CALL_H
#define RPC_CALL_H

#include <vector>

#include "RpcArchive.h"
#include "RpcSerialization.h"
#include "StlSerialization.h"

namespace rpc
{
    template <typename ExceptionsTypelist>
    class RpcCall
    {
    public:
        RpcCall(const std::string& objName, const std::string& fncName, size_t bufferSize = 16*1024):
          data_(bufferSize), curPackSize_(0), curUnpackSize_(0), answerSize_(0)
          {
              rpc::Archive sourceArchive(&data_[0], data_.size());
              rpc::Pack(sourceArchive, objName);
              rpc::Pack(sourceArchive, fncName);
              curPackSize_ = sourceArchive.GetUsedSize();
          }

          template <class senderType>
          void Call(senderType* sender)
          {
              sender->Send(&data_[0], curPackSize_, data_.size(), &answerSize_);

              rpc::Archive destArchive(&data_[0], answerSize_, answerSize_, 0);
              bool isException = false;
              rpc::Unpack(destArchive, &isException);
              curUnpackSize_ = destArchive.GetCurRange();
              if(isException)
              {
                  ExceptionPacker<ExceptionsTypelist>::UnpackAndThrow(destArchive);
              }
          }

          template <class P>
          void Pack(const P& p)
          {
              rpc::Archive sourceArchive(&data_[0], data_.size(), curPackSize_, 0);
              rpc::Pack(sourceArchive, p);
              curPackSize_ = sourceArchive.GetUsedSize();
          }
          template <class P1, class P2>
          void Pack(const P1& p1, const P2& p2)
          {
              rpc::Archive sourceArchive(&data_[0], data_.size(), curPackSize_, 0);
              rpc::Pack(sourceArchive, p1);
              rpc::Pack(sourceArchive, p2);
              curPackSize_ = sourceArchive.GetUsedSize();
          }
          template <class P1, class P2, class P3>
          void Pack(const P1& p1, const P2& p2, const P3& p3)
          {
              rpc::Archive sourceArchive(&data_[0], data_.size(), curPackSize_, 0);
              rpc::Pack(sourceArchive, p1);
              rpc::Pack(sourceArchive, p2);
              rpc::Pack(sourceArchive, p3);
              curPackSize_ = sourceArchive.GetUsedSize();
          }
          template <class P1, class P2, class P3, class P4>
          void Pack(const P1& p1, const P2& p2, const P3& p3, const P4& p4)
          {
              rpc::Archive sourceArchive(&data_[0], data_.size(), curPackSize_, 0);
              rpc::Pack(sourceArchive, p1);
              rpc::Pack(sourceArchive, p2);
              rpc::Pack(sourceArchive, p3);
              rpc::Pack(sourceArchive, p4);
              curPackSize_ = sourceArchive.GetUsedSize();
          }
          template <class P1, class P2, class P3, class P4, class P5>
          void Pack(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5)
          {
              rpc::Archive sourceArchive(&data_[0], data_.size(), curPackSize_, 0);
              rpc::Pack(sourceArchive, p1);
              rpc::Pack(sourceArchive, p2);
              rpc::Pack(sourceArchive, p3);
              rpc::Pack(sourceArchive, p4);
              rpc::Pack(sourceArchive, p5);
              curPackSize_ = sourceArchive.GetUsedSize();
          }
          template <class P1, class P2, class P3, class P4, class P5, class P6>
          void Pack(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6)
          {
              rpc::Archive sourceArchive(&data_[0], data_.size(), curPackSize_, 0);
              rpc::Pack(sourceArchive, p1);
              rpc::Pack(sourceArchive, p2);
              rpc::Pack(sourceArchive, p3);
              rpc::Pack(sourceArchive, p4);
              rpc::Pack(sourceArchive, p5);
              rpc::Pack(sourceArchive, p6);
              curPackSize_ = sourceArchive.GetUsedSize();
          }

          //unpackers
          template <class P>
          void Unpack(P* p)
          {
              rpc::Archive sourceArchive(&data_[0], answerSize_, answerSize_, curUnpackSize_);
              rpc::Unpack(sourceArchive, p);
              curUnpackSize_ = sourceArchive.GetCurRange();
          }
          template <class P1, class P2>
          void Unpack(P1* p1, P2* p2)
          {
              rpc::Archive sourceArchive(&data_[0], answerSize_, answerSize_, curUnpackSize_);
              rpc::Unpack(sourceArchive, p1);
              rpc::Unpack(sourceArchive, p2);
              curUnpackSize_ = sourceArchive.GetCurRange();
          }
          template <class P1, class P2, class P3>
          void Unpack(P1* p1, P2* p2, P3* p3)
          {
              rpc::Archive sourceArchive(&data_[0], answerSize_, answerSize_, curUnpackSize_);
              rpc::Unpack(sourceArchive, p1);
              rpc::Unpack(sourceArchive, p2);
              rpc::Unpack(sourceArchive, p3);
              curUnpackSize_ = sourceArchive.GetCurRange();
          }

    private:
        std::vector<char> data_; 
        size_t curPackSize_;
        size_t curUnpackSize_;
        size_t answerSize_;
    };
}// namespace rpc

#endif