#ifndef RPC_SERVER_H
#define RPC_SERVER_H

#include "new"
#include "vector"
#include "exception"

#include "RpcArchive.h"
#include "RpcSerialization.h"
#include "StlSerialization.h"
#include "RpcServerInterfaces.h"

namespace rpc
{
    class RpcServerRequest
    {
    public:
        RpcServerRequest(rpc::IRpcSkel* obj, const std::string& fncName, rpc::Archive& inBuf, rpc::Archive& outBuf)
            : obj_(obj)
            , fncName_(fncName)
            , inBuf_(inBuf)
            , outBuf_(outBuf)
        {}

        void Process()
        {
            // Exception flag - no exceptions
            rpc::Pack(outBuf_, false);
            obj_->Call(fncName_, inBuf_, outBuf_);
        }

        void operator()()
        {
            return Process();
        }

    private:
        RpcServerRequest(const RpcServerRequest&);
        RpcServerRequest& operator=(const RpcServerRequest&);

    private:
        IRpcSkel* obj_;
        const std::string& fncName_;
        rpc::Archive& inBuf_;
        rpc::Archive& outBuf_;
    };

    //////////////////////////////////////////////////////////////////////////

    class RpcServerImpl: public IRpcServer
    {
    public:
        void RegisterObject(const std::string& objName, IRpcSkel* pObject);
        void UnregisterObject(const std::string& objName);

        IRpcSkel* FindObject(const std::string& objName);

    private:
        typedef std::map<std::string, IRpcSkel*> RpcObjectMap;
        RpcObjectMap objects_;
    };

    template <typename ExceptionsTypelist>
    class RpcServer: public RpcServerImpl
    {
    public:
        void Process(char* buf, size_t inBufSize, size_t maxBufSize, size_t* outBufSize)
        {
            try
            {
                Archive input(buf, inBufSize, inBufSize, 0);
                Archive output(buf, maxBufSize);

                std::string objName, fncName;
                rpc::Unpack(input, &objName);
                rpc::Unpack(input, &fncName);

                rpc::RpcServerRequest request(FindObject(objName), fncName, input, output);
                ExceptionPacker<ExceptionsTypelist>::Dispatch(output, request);
                *outBufSize = output.GetUsedSize();
            }
            catch(const std::exception& ex)
            {
                Archive data(buf, maxBufSize);
                rpc::Pack(data, true);//exception handled flag
                rpc::ExceptionPacker<ExceptionsTypelist>::Pack(data, ex);
                *outBufSize = data.GetUsedSize();
            }
            catch(...)
            {
                Archive data(buf, maxBufSize);
                rpc::Pack(data, true);//exception handled flag
                rpc::ExceptionPacker<ExceptionsTypelist>::Pack(data, std::runtime_error("unhandled exception"));
                *outBufSize = data.GetUsedSize();
            }
        }
    };
}
#endif