#ifndef RPC_SKEL_H
#define RPC_SKEL_H

#include "new"
#include "map"

#include <boost/bind.hpp>
#include <boost/function.hpp>

#include "RpcServer.h"

namespace rpc
{
    class RpcSkelBase : public rpc::IRpcSkel
    {
        typedef boost::function<void(rpc::Archive&, rpc::Archive&)> RpcImplMethodType;
        typedef std::map<std::string, RpcImplMethodType> RpcMethodMap;

    public:
        void Call(const std::string& fncName, rpc::Archive& inBuf, rpc::Archive& outBuf)
        {
            RpcMethodMap::iterator it = rpcMethods_.find(fncName);
            if (it == rpcMethods_.end())
                throw std::runtime_error(std::string(__FUNCTION__) + " - method " + fncName + " not registered");

            RpcImplMethodType pMethod = it->second;
            pMethod(inBuf, outBuf);
        }

        void RegisterRpcFunction(const std::string& funcName, RpcImplMethodType pFunc)
        {
            if (!rpcMethods_.insert(std::make_pair(funcName, pFunc)).second)
            {
                throw std::runtime_error(std::string(__FUNCTION__) + " - method " + funcName + " already registered");
            }
        }

    private:
        RpcMethodMap rpcMethods_;
    };
}

#define RPC_FUNCTION_EX(skel, srvObj, srvClass, rpcFunc) \
    skel->RegisterRpcFunction( \
        #rpcFunc, \
        boost::bind(&srvClass::rpcFunc, srvObj, _1, _2) \
    )

#define RPC_FUNCTION(srvClass, rpcFunc) RPC_FUNCTION_EX(this, this, srvClass, rpcFunc)

#endif
