#ifndef RPC_TYPES_H
#define RPC_TYPES_H

#include "stdexcept"
#include "memory"

#include "vector"

#include "RpcTypelist.h"

namespace rpc
{
    typedef long long rpc_size_t;

    //////////////////////////////////////////////////////////////////////////
    // RID's:
    //    [   0.. 1024] - reserved for basic types
    //    [1024.. ****] - user-defined types

#define RPC_USER_RID(relative_id) (1024 + relative_id)

    //////////////////////////////////////////////////////////////////////////
    // BASIC TYPES

    const int rid_unknown = 0;

    // native
    const int rid_bool = 10;
    const int rid_rpc_size_t = 11;
    const int rid_char = 12;
    const int rid_unsigned_char = 13;
    const int rid_short = 14;
    const int rid_unsigned_short = 15;
    const int rid_int = 16;
    const int rid_unsigned_int = 17;
    const int rid_long = 18;
    const int rid_unsigned_long = 19;
    const int rid_long_long = 20;
    const int rid_unsigned_long_long = 21;
    const int rid_size_t = 22;
    const int rid_wchar_t = 23;
    const int rid_float = 24;
    const int rid_double = 25;

    const int rid_ptr_void = 101;

    // stl containers
    const int rid_vector_char = 201;
    const int rid_basic_string = 202;
    const int rid_map = 203;
    const int rid_sequence = 204;

    // exceptions
    const int rid_std_exception = 601;
    const int rid_std_bad_alloc = 602;
    const int rid_std_bad_cast = 603;
    const int rid_std_bad_exception = 604;
    const int rid_std_bad_typeid = 605;
    const int rid_std_runtime_error = 606;
    const int rid_std_range_error = 607;
    const int rid_std_overflow_error = 608;
    const int rid_std_underflow_error = 609;
    const int rid_std_logic_error = 610;
    const int rid_std_domain_error = 611;
    const int rid_std_invalid_argument = 612;
    const int rid_std_length_error = 613;
    const int rid_std_out_of_range = 614;

    //////////////////////////////////////////////////////////////////////////

    template<class Type>
    struct resource_traits
    {
        enum { };
    };

    //////////////////////////////////////////////////////////////////////////

#define DEFINE_RESOURCE_EX(ResTypeName, ResId) \
    template<> struct resource_traits<ResTypeName>  {    enum { ResourceId = ResId };  };

#define DEFINE_RESOURCE_PTR(ResTypeName) \
    DEFINE_RESOURCE_EX(const ResTypeName*, rid_ptr_##ResTypeName) \
    DEFINE_RESOURCE_EX(ResTypeName*, rid_ptr_##ResTypeName)

#define DEFINE_RESOURCE(ResTypeName) DEFINE_RESOURCE_EX(ResTypeName, rid_##ResTypeName)

    //////////////////////////////////////////////////////////////////////////

    // native
    DEFINE_RESOURCE     (bool);
    DEFINE_RESOURCE     (char);
    DEFINE_RESOURCE_EX  (unsigned char, rid_unsigned_char);
    DEFINE_RESOURCE     (short);
    DEFINE_RESOURCE_EX  (unsigned short, rid_unsigned_short);
    DEFINE_RESOURCE     (int);
    DEFINE_RESOURCE_EX  (unsigned int, rid_unsigned_int);
    DEFINE_RESOURCE     (long);
    DEFINE_RESOURCE_EX  (unsigned long, rid_unsigned_long);
    DEFINE_RESOURCE_EX  (long long, rid_long_long);
    DEFINE_RESOURCE_EX  (unsigned long long, rid_unsigned_long_long);
    DEFINE_RESOURCE     (wchar_t);
    DEFINE_RESOURCE     (float);
    DEFINE_RESOURCE     (double);
    DEFINE_RESOURCE_PTR (void);

    //////////////////////////////////////////////////////////////////////////

    // typelist of standard exceptions

    typedef
        TL::MakeTypelist
        <
        std::exception
        , std::bad_alloc
        , std::bad_cast
        , std::bad_exception
        , std::bad_typeid
        , std::runtime_error
        , std::range_error
        , std::overflow_error
        , std::underflow_error
        , std::logic_error
        , std::domain_error
        , std::invalid_argument
        , std::length_error
        , std::out_of_range
        >::Result
        DefaultExceptions;

    //////////////////////////////////////////////////////////////////////////

    template<class Type>
    struct serialize_traits
    {
    };
}

#endif
