#include "RpcServerTest.h"

BOOST_AUTO_TEST_SUITE(RpcLibTest)

    BOOST_FIXTURE_TEST_CASE(RegisterObject, RpcServerFix)
    {
        TestSkel testObj;
        rpcServer.RegisterObject("TestObject", &testObj);

        BOOST_REQUIRE_EQUAL(rpcServer.FindObject("TestObject"), &testObj);
    }

    BOOST_FIXTURE_TEST_CASE(UnregisterObject, RpcServerFix)
    {
        TestSkel testObj;
        rpcServer.RegisterObject("TestObject", &testObj);
        BOOST_REQUIRE_EQUAL(rpcServer.FindObject("TestObject"), &testObj);

        rpcServer.UnregisterObject("TestObject");
        BOOST_REQUIRE_THROW(rpcServer.FindObject("TestObject"),std::runtime_error);
    }

    BOOST_FIXTURE_TEST_CASE(RegisterTwoObjects, RpcServerFix)
    {
        TestSkel testObj1, testObj2;
        rpcServer.RegisterObject("TestObject1", &testObj1);
        rpcServer.RegisterObject("TestObject2", &testObj2);

        BOOST_REQUIRE_EQUAL(rpcServer.FindObject("TestObject1"), &testObj1);
        BOOST_REQUIRE_EQUAL(rpcServer.FindObject("TestObject2"), &testObj2);
    }

    BOOST_FIXTURE_TEST_CASE(RegisterObjectsWithSameName, RpcServerFix)
    {
        TestSkel testObj1, testObj2;
        rpcServer.RegisterObject("TestObject", &testObj1);

        BOOST_REQUIRE_THROW(rpcServer.RegisterObject("TestObject", &testObj2), std::runtime_error);
    }

    BOOST_FIXTURE_TEST_CASE(Process, RpcServerFix)
    {
        TestSkel testObj;
        rpcServer.RegisterObject("TestObject", &testObj);

        std::vector<char> buffer(256);
        rpc::Archive input(&buffer.front(), buffer.size());

        rpc::Pack(input, std::string("TestObject"));
        rpc::Pack(input, std::string("TestMethod"));

        const int inputParam = 5;
        rpc::Pack(input, inputParam);

        size_t outBufSize;
        rpcServer.Process(&buffer.front(), buffer.size(), buffer.capacity(), &outBufSize);
        buffer.resize(outBufSize);

        BOOST_REQUIRE(outBufSize > 0);

        rpc::Archive output(&buffer.front(), buffer.size(), buffer.size());

        bool exceptionFlag;
        rpc::Unpack(output, &exceptionFlag);

        BOOST_REQUIRE(!exceptionFlag);

        int outputParam;
        rpc::Unpack(output, &outputParam);
    }

    BOOST_FIXTURE_TEST_CASE(ProcessException, RpcServerFix)
    {
        TestSkel testObj;
        rpcServer.RegisterObject("TestObject", &testObj);

        std::vector<char> buffer(256);
        rpc::Archive input(&buffer.front(), buffer.size());

        rpc::Pack(input, std::string("TestObject"));
        rpc::Pack(input, std::string("WrongMethod"));

        size_t outBufSize;
        rpcServer.Process(&buffer.front(), buffer.size(), buffer.capacity(), &outBufSize);
        buffer.resize(outBufSize);

        BOOST_REQUIRE(outBufSize > 0);

        rpc::Archive output(&buffer.front(), buffer.size(), buffer.size());

        bool exceptionFlag;
        rpc::Unpack(output, &exceptionFlag);

        BOOST_REQUIRE(exceptionFlag);
    }

BOOST_AUTO_TEST_SUITE_END()
