#include "RpcTest.h"

#include <vector>

BOOST_AUTO_TEST_SUITE(RpcLibTest)

    BOOST_FIXTURE_TEST_CASE(TestValidity, RpcTestFix)
    {
        rpc::RpcCall<rpc::DefaultExceptions> rpcCall("test_object", "TestValidityMethod");

        int param = 10;
        rpcCall.Pack(param);
        rpcCall.Call(&transport_);

        int outParam;
        rpcCall.Unpack(&outParam);
        BOOST_REQUIRE_EQUAL(param + 1, outParam);
    }
    BOOST_FIXTURE_TEST_CASE(TestValidityWithoutParams, RpcTestFix)
    {
        rpc::RpcCall<rpc::DefaultExceptions> rpcCall("test_object", "TestValidityWithoutParams");
        rpcCall.Call(&transport_);
    }
    BOOST_FIXTURE_TEST_CASE(TestException, RpcTestFix)
    {
        rpc::RpcCall<rpc::DefaultExceptions> rpcCall("test_object", "TestExceptionMethod");
        BOOST_REQUIRE_THROW(rpcCall.Call(&transport_), std::runtime_error);
    }

    BOOST_FIXTURE_TEST_CASE(TestString, RpcTestFix)
    {
        rpc::RpcCall<rpc::DefaultExceptions> rpcCall("test_object", "TestStringMethod");

        std::string param("test_");
        rpcCall.Pack(param);
        rpcCall.Call(&transport_);

        std::string outString;
        rpcCall.Unpack(&outString);

        std::string resultString("test_succeeded");
        BOOST_REQUIRE_EQUAL(outString, resultString);
    }

    BOOST_FIXTURE_TEST_CASE(TestVector, RpcTestFix)
    {
        rpc::RpcCall<rpc::DefaultExceptions> rpcCall("test_object", "TestVectorMethod");

        std::vector<int> dataVector;
        dataVector.push_back(25);
        dataVector.push_back(34);
        dataVector.push_back(234);
        dataVector.push_back(745);
        dataVector.push_back(1043);

        int old_sum = 0;
        for(std::vector<int>::iterator it = dataVector.begin(); it != dataVector.end(); ++it)
            old_sum += *it;

        rpcCall.Pack(dataVector);
        rpcCall.Call(&transport_);

        std::vector<int> outVector;
        rpcCall.Unpack(&outVector);

        int new_sum = 0;
        for(std::vector<int>::iterator it = outVector.begin(); it != outVector.end(); ++it)
            new_sum += *it;

        BOOST_REQUIRE_EQUAL(old_sum + 5, new_sum);
    }

    BOOST_FIXTURE_TEST_CASE(TestMap, RpcTestFix)
    {
        rpc::RpcCall<rpc::DefaultExceptions> rpcCall("test_object", "TestMapMethod");

        typedef std::map<std::string, int> TestMap;

        TestMap mapParam;
        mapParam["test_item"] = 10;
        rpcCall.Pack(mapParam);
        rpcCall.Call(&transport_);

        TestMap outMapParam;
        rpcCall.Unpack(&outMapParam);

        TestMap::iterator it = outMapParam.find("new_item");
        if(it == outMapParam.end())
            BOOST_FAIL("test falied");

        BOOST_REQUIRE_EQUAL(it->second, ++mapParam["test_item"]);
    }

BOOST_AUTO_TEST_SUITE_END()
